require 'sketchup.rb'
menu_name = "Unfold Tool"
unless file_loaded?(menu_name)
    UI.menu("Plugins").add_item(menu_name) { Sketchup.active_model.select_tool(@uft ||= UnfoldTool.new) } 
    #UI.menu("Plugins").add_item("Re Fold") { @uft.refold }	
    file_loaded(menu_name)
end
class UnfoldTool
    def initialize
        @selected_faces = []
        cursor = File.join(File.dirname(__FILE__), "unfoldtool.png")
        @cursor = UI.create_cursor(cursor, 13, 16)
        @history = []
    end
    def activate
        @msg = "<<Unfold selected to next>>  Ctrl = Match next to selected  Shift = Toggle selection  Alt = Select plane"
        Sketchup.set_status_text "Unfold: Select face to unfold  |  Shift = toggle, Alt = select plane. "
        @selected_faces = Sketchup.active_model.selection.find_all{|e| e.class == Sketchup::Face}
        if @history.length > 0
            #if(UI.messagebox("Clear History?", MB_YESNO) == IDYES)
                @history = []
            #end
        end
    end
    def onSetCursor
        UI.set_cursor(@cursor)
    end
    def onLButtonDown(flags, x, y, view)
        ph = Sketchup.active_model.active_view.pick_helper
        ph.do_pick x, y
        ref = ph.best_picked
        if ref.nil?
            puts "ref.nil"
            Sketchup.active_model.selection.clear
            @selected_faces = []
            Sketchup.set_status_text "Unfold: Selection cleared. Select face to unfold."
            return
        end
        if( ref.typename == "Face" )
            if flags == 5 or selected_faces().length == 0 # Shift key
                Sketchup.active_model.selection.toggle ref
                @selected_faces.push(ref) #unless flags == 9
                Sketchup.set_status_text @msg
                return
            end
            Sketchup.active_model.start_operation "Unfold"
            st = unfold(ref,flags == 9) # Ctrl = 9
            if st
                Sketchup.active_model.selection.add ref
                Sketchup.active_model.selection.add selected_faces()
            end
        end
        # if face picked
        if ref.typename == "Edge"
        end
        # if edge picked
        Sketchup.active_model.commit_operation
    end
    # onLeftButtonDown
    def onLButtonDoubleClick(flags, x, y, view)
        ph = Sketchup.active_model.active_view.pick_helper
        ph.do_pick x, y
        ref = ph.best_picked
        @selected_faces = select_plane(ref.plane)
    end
    def onKeyDown(key, repeat, flags, view)
        txts=[ "  Unfold selected to next", "Ctrl = Match next to selected",
        "Shift = Toggle selection", "Alt = Select plane" ]
        case key
        when 17 # Ctrl
            txts[1] = "<<" + txts[1] + ">>"
        when 16 # Shift
            txts[2] = "<<" + txts[2] + ">>"
        when 18 # Alt
            txts[3] = "<<" + txts[3] + ">>"
        end
        Sketchup.set_status_text txts.join("  ")
    end
    def onKeyUp(key, repeat, flags, view)
        Sketchup.set_status_text @msg
    end
    def deactivate(*args)
    end
    def resume(view)
        if selected_faces().length != 0
            msg = @msg
        else
            msg = "Unfold: Select face to unfold."
        end
        Sketchup.set_status_text msg
    end
    def self.uselect
        if @selected_faces.length > 1
            face = @selected_faces.pop
            Sketchup.active_model.selection.remove(face)
        end
    end

    def unfold(ref, rev)
        #curr_sel = Sketchup.active_model.selection.find_all{|e| e.class == Sketchup::Face}
        if rev
            to_group = ref
        else
            to_group = selected_faces()
        end
        n1 = ref.normal
        face = @selected_faces[-1]
        n2 = face.normal
        angle = n1.angle_between n2 
        angle = -angle if rev
        axis = Geom::intersect_plane_plane(ref.plane, face.plane)
        unless axis
            puts "No axis"
            return
        end
        g = Sketchup.active_model.entities.add_group(to_group)
        tr = Geom::Transformation.rotation(axis[0], axis[1], -angle)
        @history.push( [ tr, to_group ] )
        g.transform!(tr)
        ents = g.explode
        ents = ents.select{|e| e.is_a? Sketchup::Drawingelement}
        Sketchup.active_model.selection.add ents
    end

    def refold()
        puts "no history" if @history.empty?
        Sketchup.active_model.start_operation("Refold")
        @history.reverse.each do |pair|
            tr, faces = pair
            g = Sketchup.active_model.entities.add_group(faces)
            g.transform!(tr.inverse)
            g.explode
        end
        Sketchup.active_model.commit_operation
    end

    def selected_faces
        Sketchup.active_model.selection.select { |e| e.typename == "Face" }
    end

    def self.select_plane(plane)
        factor = 10000
        faces = Sketchup.active_model.active_entities.select {|e| e.typename=="Face"}
        flag = true
        plane = plane.map { |e| (e*factor).to_i }
        faces.each do |face|
            flag = true
            fplane = face.plane.map { |e| (e*factor).to_i }
            if plane != fplane
                flag = false
            end
            if flag
                Sketchup.active_model.selection.add face
            end
        end
        faces = Sketchup.active_model.selection.collect
        faces
    end
end
#class UnfoldTOol
